/**
 * Created by John  on 7/11/2017.
 */
var express = require('express');
var passport = require('passport');

var app = express();

//make Sequelize a singleton
app.set('models', require('./components/models'));
var models = app.get('models');

var port = process.env.PORT || 3414;

var env         = process.env.NODE_ENV || "dev";
var config      = require("./components/config/config.json")[env];

//initialize logging
var winston = require('winston');
var win = require('./components/config/logging')(winston, config);
app.set('winston', win);
var logger = win.loggers.get("main");

app.listen(port, '0.0.0.0');
logger.info("------------> M3-API START ! <------------------");
logger.info("Pid : " +process.pid + " - Fresenius Console API Server started on port " + port + " on " + env);
logger.info("Rabbit Topology");
logger.info("       host -> " + config.rabbitConfig.host);
logger.info("       user -> " + config.rabbitConfig.user);
logger.info("   password -> " + config.rabbitConfig.pass);
logger.info("       port -> " + config.rabbitConfig.port);
logger.info("      vhost -> " + config.rabbitConfig.vhost);
//logger.info("Rabbit Topology set - config : " + JSON.stringify(config.rabbitConfig, null, 4));

//initialize passport
require('./components/config/passport')(passport, models);

//initialize express
require('./components/config/express')(app, passport);

//initialize routes
require('./routes/api')(app, passport, models);
//require('./routes/test')(app, passport, models);
require('./routes/web')(app, passport, models);

// Initialize Rabbit MQ
require('./innerQueues')(winston, models, config);

//initialize connections to the MQ framework
if (config.mqs.mqs !== null && config.mqs.mqs === "rabbit") {
    logger.info("--> Initialize Rabbit MQ");
    var Rabbit = require("./components/helpers/RabbitHelper");
    app.set("mqs", new Rabbit(winston, config));
} else {
    logger.info("--> Initialize Rabbit MQ for Squiss");
    var SquissHelper = require("./components/helpers/SqsHelper");
    app.set("mqs", new SquissHelper(winston, config));
}

process.on('uncaughtException', function (err) {
    var msg = (new Date).toUTCString() + ' uncaughtException:';
    logger.error(msg, err.message);
    logger.error(err.stack);
    process.exit(1);
});

module.exports = app;
