/**
 * Created by TFG on 07/01/20.
 *
 * This contains the OLD management for the AWCBL loading operations.
 * Only ONE upload file at a time is allowed here !
 *
 * CAUTION !! THIS CODE IS KEPT FOR NOW ONLY TO DOCUMENT HOW THE ORIGINAL BULK LOAD WAS DONE
 * THIS CODE IS NOT IN USE ANYMORE !!
 */

var Promise = require('promise');
var fs = require('fs');
var awcblcommon = require('./awcblcommon');

var expectedExecutionTime = 120000;     // Expected execution time (watchdog) in ms (default 2 minutes - 120 sec - 120000 ms)
var loadingPromise=null;                // Store the promise for the load operations !Variable for Master Items loading promise
var processedCounter=0;                 // Counter used to determine if all the loads were fullfilled

var loadingMIPromise=null;              // Variable for Master Items loading promise
var loadingCPPromise=null;              // Variable for Clinic Profile promise
var loadingPVPromise=null;              // Variable for Par Value promise
var loadingRecPromise=null;             // Variable for Par Value promise
var loadingCRPromise=null;              // Variable for Clinic Routing promise

var debug = true;               // Flag to handle extra debug messages

//
// ------------------- Exporting functions --------------------------------------
//

module.exports.loadMasterItem = loadMasterItem;         // Used from filewatcher.js
module.exports.loadItemsParValue = loadItemsParValue;   // Used from filewatcher.js
module.exports.loadClinicProfile = loadClinicProfile;   // Used from filewatcher.js

//
// ------------------- Master Items Bulk management -----------------------------
//

// Entry point loading management
function loadMasterItem (app, file) {
    var winston = app.get('winston');
    var logger = winston.loggers.get('main');
    var transactionLogger = winston.loggers.get('transaction');
    var models = app.get('models');

    logger.debug("loadMasterFile on " + file.name);
    fs.readFile(file, 'utf8', function (err, data) {
        if (err) {
            logger.error("Reading file error : " + err.toString());
        } else {
            if (readMasterItems(data) === -1) {
                logger.error("Parsing data errors ! Aborted !");
            } else {
                // Parsing correct and completed !
                // Move the file in archive directory
                awcblcommon.archiveFile(app, file, data);

                // Reset all the promises here. Only one upload should be allowed at any moment
                // In the end probably only one promise will be used for all the loads - for now erase all of them
                loadingMIPromise = null;
                loadingCPPromise = null;
                loadingPVPromise = null;
                loadingCRPromise = null;

            }
        }
    });

    function readMasterItems (data) {
        logger.debug("readMasterItems : " + data);
        if (loadingMIPromise && loadingMIPromise.toString() !== data) {
            loadingMIPromise = null;
        }

        if (loadingMIPromise) {
            return loadingMIPromise;
        } else {
            // cache this promise so any other requests while this one is stil
            // in flight will use the same promise
            loadingMIPromise = new Promise(function (resolve, reject) {
                var allCatalogs = [];
                var lines = data.split(/\r?\n/);
                var masterItems = [];

                models.CatalogList.findAll().then(function (catalogs) {
                    if (catalogs) {
                        allCatalogs = catalogs;
                    }

                    for (var i = 0; i < lines.length; i++) {
                        var line = lines[i].split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);

                        line = line || [];

                        if (line.length > 1) {
                            var testCatalog = line[4].split('"').join('');

                            if (!testCatalog) {
                                logger.debug("Catalog name missing ! Forced to empty (skip check)");
                                testCatalog = "";
                            }

                            if (testCatalog !== "") {
                                logger.debug("Looking for catalog [" + testCatalog + "] " + "for product : " + parseInt(line[0]));

                                //we need to convert names to id's
                                var fileVendorNames = testCatalog.split(",");
                                testCatalog = "";
                                var cnt = 0;
                                for (var fn = 0; fn < fileVendorNames.length; fn++) {
                                    var id = awcblcommon.getCatalogId(app, allCatalogs, fileVendorNames[fn].trim());
                                    if (id >= 0) {
                                        logger.debug("Found catalog [" + fileVendorNames[fn].trim() + "] with index " + id);

                                        testCatalog = testCatalog + ((cnt > 0) ? "," : "") + id;
                                        cnt++;
                                    } else {
                                        logger.error("Catalog parsing Error on [" + fileVendorNames[fn] + "] - NOT FOUND ! Abort !");
                                        loadingMIPromise = null;
                                        return -1;
                                    }
                                }
                            }

                            // CAUTION! Discontinued field note !
                            // The discontinued field contains a date indicating the item as discontinued.
                            // The systems allows only to add a discontinued date, NOT to remove it.
                            //   - If the item in the db has null in the discontinued date, it will overwrite it with the
                            //     discontinued date loaded here.
                            //   - If the item in the db has a discontinued date set and the field here is null,
                            //     nothing will happens (i.e. the discontinued date in the db will remains
                            //   - If the item in the db has a discontinued date set and the field here contains a date,
                            //     the discontinued date in the DB will be overwritten
                            // Currently the only way to force to null a discontinued field in the DB is to do that
                            // manually !

                            try {
                                var masterItem = models.MasterItem.build({
                                    sapMaterial: parseInt(line[0]),
                                    description: line[1].split('"').join(''),
                                    cntInd: line[2].split('"').join('') === 'X',
                                    cntDescription: line[3].split('"').join(''),
                                    medVendor: testCatalog,
                                    vendorMaterial: line[5].split('"').join(''),
                                    conv: parseInt(line[6].split('"').join('')),
                                    countType: line[7].split('"').join(''),
                                    alternates: line[8].split('"').join(''),
                                    unitPrice: awcblcommon.formatPrice(app, line[9].split('"').join('')),
                                    vendorId: line[10].split('"').join(''),
                                    ndc: line[11].split('"').join(''),
                                    showInventoryInd: line[12].split('"').join('') === 'X',
                                    discontinued: awcblcommon.formatDate(app, line[13].split('"').join('')),
                                    dateModified: new Date()
                                });

                                if (isNaN(masterItem.conv)) {
                                    masterItem.conv = 1;
                                }

                                masterItems.push(masterItem);

                            } catch (ex) {
                                logger.error("readMasterItems Error : " + ex + " - Abort !");
                                loadingMIPromise = null;
                                return -1;
                            }
                        }
                    }
                    resolve(bulkUpsertMasterItems(masterItems));
                });
            });
            loadingMIPromise.__proto__.toString = function () {
                return data;
            };
            return loadingMIPromise;
        }
    }

    function bulkUpsertMasterItems (masterItems) {
        var promises = [];

        masterItems.forEach(function (masterItem) {
            promises.push(upsertMasterItem(masterItem));
        });

        return models.sequelize.Promise.all(promises);
    }

    // Function to update the catalogs assignment associated to the tbl_master_item just written
    //
    // find the record or create it
    //if "deleted", update anyway, but leave the deleted flag on  (to skip altogether, remove the paranoid:false option)
    //
    function upsertMasterItem (masterItem) {
        return models.MasterItem.findOrCreate({
            where: {
                sapMaterial: masterItem.sapMaterial,
                vendorMaterial: masterItem.vendorMaterial
            }, raw: true, paranoid: false
        }).spread(function (itemRecord, created) {
          masterItem.id = itemRecord.id;
          models.MasterItem.update(masterItem.dataValues, { where: { id: masterItem.id } })
            .then(function (count) {
                if (count > 0 && masterItem.medVendor) {
                    var ids = masterItem.medVendor.split(",");

                    models.ItemCatalog.destroy({ where: { itemId: masterItem.id } }).then(function (num) {
                        var cats = [];
                        for (var i = 0; i < ids.length; i++) {
                            var cat = models.ItemCatalog.build({
                                itemId: masterItem.id,
                                catalogId: ids[i]
                            });
                            cats.push(cat);
                        }

                        if (cats.length > 0) {
                            var catpromises = [];

                            cats.forEach(function (cat) {
                                catpromises.push(cat.save());
                            });

                            models.sequelize.Promise.all(catpromises).then(function (vals) {
                                transactionLogger.info("Master Item " + ((created) ? "created" : "updated") + " by ftp syc: " + JSON.stringify(masterItem));
                                logger.info("Master Item " + ((created) ? "created" : "updated") + " by ftp syc: " + JSON.stringify(masterItem));
                                return cats.length;
                            }).catch(function (saveErr) {
                                logger.error(saveErr);
                                return -1;
                            });
                        }
                    }).catch(function (delerr) {
                        logger.error(delerr);
                        return -1;
                    });
                } else {
                    transactionLogger.info("Master Item " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(masterItem));
                    logger.info("Master Item " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(masterItem));
                    return count;
                }
            }).catch(function (upderr) {
              transactionLogger.error(upderr);
              logger.error(upderr);
              return -1;
          });
      }).catch(function (err) {
          logger.error(err);
          return -1;
      });
    }
}

//
// ------------------- Clinic Profile Bulk management -----------------------------
//

function loadClinicProfile(app, file) {
    var winston = app.get('winston');
    var logger = winston.loggers.get('main');
    var transactionLogger = winston.loggers.get('transaction');
    var models = app.get('models');

    fs.readFile(file, 'utf8', function (err, data) {
        if (err) {
            logger.error(err);
        } else {
            // Parsing correct and completed !
            // Move the file in archive directory
            readClinicProfiles(data);
            awcblcommon.archiveFile(app, file, data);

            // Reset all the promises here. Only one upload should be allowed at any moment
            // In the end probably only one promise will be used for all the loads - for now erase all of them
            loadingCPPromise = null;  // Reset here
            loadingMIPromise = null;
            loadingPVPromise = null;
            loadingCRPromise = null;
        }
    });

    function readClinicProfiles (data) {
        logger.debug("readClinicProfile : " + data);

        if (loadingCPPromise && loadingCPPromise.toString() !== data) {
            loadingCPPromise = null;
        }
        if (loadingCPPromise) {
            return loadingCPPromise;
        } else {
            // cache this promise so any other requests while this one is still
            // in flight will use the same promise
            loadingCPPromise = new Promise(function (resolve, reject) {
                var allCatalogs = [];
                var clinicProfiles = [];
                var strDefCatalogs = "";    // String containg the default catalog entry ready to add
                models.CatalogList.findAll().then(function (catalogs) {
                      if (catalogs) {
                          allCatalogs = catalogs;
                      }

                      for (var vl = 0; vl < allCatalogs.length; vl++) {
                          var dv = allCatalogs[vl];
                          if (dv.isDefault === true) {
                              strDefCatalogs += "," + dv.id;
                          }
                      }
                      logger.debug("Default catalog Id for the entry : " + strDefCatalogs.substr(1));

                      clinicProfiles = buildClinicProfileObjects(data, allCatalogs, strDefCatalogs);
                      resolve(bulkUpsertClinicProfiles(clinicProfiles));
                  },
                  function (err) {
                      reject(err);
                  });
            });

            loadingCPPromise.__proto__.toString = function () {
                return data;
            };
            return loadingCPPromise;
        }
    }

    function buildClinicProfileObjects (data, allCatalogs, strDefCatalogs) {
        var clinicProfiles = [];
        var lines = data.split(/\r?\n/);

        for (var i = 0; i < lines.length; i++) {
            var line = lines[i].split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);
            if (line.length > 1) {
                var clinicId = line[0];
                if (clinicId.length === 4) {
                    clinicId = "." + clinicId + "BM";
                } else {
                    if (!clinicId.startsWith(".")) {
                        clinicId = "." + clinicId;
                    }
                }
                var testCatalog = line[1].split('"').join('');

                if (!testCatalog) {
                    testCatalog = "";
                }

                if (testCatalog !== "") {
                    var isDefaultPresent = false;

                    //we need to convert names to id's
                    var fileVendorNames = testCatalog.split(",");
                    testCatalog = "";
                    var cnt = 0;
                    for (var fn = 0; fn < fileVendorNames.length; fn++) {
                        var id = awcblcommon.getCatalogId(app, allCatalogs, fileVendorNames[fn].trim());
                        if (id >= 0) {
                            if (id.toString() === strDefCatalogs.substr(1)) {
                                logger.debug("Found default catalog !");
                                isDefaultPresent = true;
                            } else {
                                logger.debug("Found catalog [" + fileVendorNames[fn].trim() + "] with index " + id);
                            }

                            testCatalog = testCatalog + ((cnt > 0) ? "," : "") + id;
                            cnt++;
                        } else {
                            logger.error("Catalog parsing Error on [" + fileVendorNames[fn] + "] - NOT FOUND !");
                        }
                    }

                    // If the default catalog is NOT present, add it to the list
                    if (isDefaultPresent === false) {
                        logger.debug("Add Default catalog to the list");
                        testCatalog += strDefCatalogs;
                    }
                } else {
                    // No catalogs found - add at least the default one
                    testCatalog = strDefCatalogs.substr(1);
                }

                try {
                    var clinicProfile = models.ClinicProfile.build({
                        clinicId: clinicId,
                        vendorList: testCatalog,
                        mirceraVisible: line[2].split('"').join('') === 'y',
                        mirceraIo: line[3].split('"').join('') === 'y',
                        areanespVisible: line[4].split('"').join('') === 'y',
                        areanespIo: line[5].split('"').join('') === 'y',
                        epogenVisible: line[6].split('"').join('') === 'y',
                        epogenIo: line[7].split('"').join('') === 'y',
                        retacritVisible: line[8].split('"').join('') === 'y',
                        retacritIo: line[9].split('"').join('') === 'y',
                        countInEach: line[10].split('"').join('') === 'y',
                        dateModified: new Date()
                    });

                    clinicProfiles.push(clinicProfile);
                } catch (ex) {
                    logger.error("buildClinicProfile Error : " + ex);
                }
            }
        }

        return clinicProfiles;
    }

    function bulkUpsertClinicProfiles (clinicProfiles) {
        var promises = [];

        clinicProfiles.forEach(function (clinicProfile) {
            promises.push(upsertClinicProfile(clinicProfile));
        });

        return models.sequelize.Promise.all(promises);
    }

// Function to update the catalogs assignment associated to the tbl_clinic_profile just written
//
    function upsertClinicProfile (clinicProfile) {

        return models.ClinicProfile.findOrCreate({
            where: {
                clinicId: clinicProfile.clinicId
            }, raw: true, paranoid: false
        })
          .spread(function (clinicRecord, created) {
              clinicProfile.id = clinicRecord.id;

              logger.debug("In tbl_clinic_profile found for " + clinicProfile.clinicId + " the Id " + clinicProfile.id);

              models.ClinicProfile.update(clinicProfile.dataValues, { where: { id: clinicProfile.id } })
                .then(function (count) {
                    if (count > 0 && clinicProfile.vendorList) {
                        var ids = clinicProfile.vendorList.split(",");

                        models.ClinicCatalog.destroy({ where: { clinicId: clinicProfile.id } }).then(function (num) {
                            var cats = [];
                            for (var i = 0; i < ids.length; i++) {
                                var cat = models.ClinicCatalog.build({
                                    clinicId: clinicProfile.id,
                                    catalogId: ids[i]
                                });
                                cats.push(cat);
                            }

                            if (cats.length > 0) {
                                var catpromises = [];

                                cats.forEach(function (cat) {
                                    catpromises.push(cat.save());
                                });

                                models.sequelize.Promise.all(catpromises).then(function (vals) {
                                    transactionLogger.info("Clinic profile " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(clinicProfile));
                                    logger.info("Clinic profile " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(clinicProfile));
                                    return cats.length;
                                }).catch(function (saveErr) {
                                    logger.error(saveErr);
                                    return -1;
                                });
                            }
                        }).catch(function (delerr) {
                            logger.error(delerr);
                            return -1;
                        });
                    } else {
                        transactionLogger.info("Clinic profile " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(clinicProfile));
                        logger.info("Clinic profile " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(clinicProfile));

                        return count;
                    }
                }).catch(function (upderr) {
                  transactionLogger.error(upderr);
                  logger.error(upderr);
                  return -1;
              });
          }).catch(function (err) {
              logger.error(err);
              return -1;
          });
    }
}

//
// ------------------- Items par value Bulk management --- FRES-129 ------------------------
//

// Load and prepare to read item par value input file
// The function expect as input the path + file name to upload.
function loadItemsParValue(app, file) {
    var winston = app.get('winston');
    var logger = winston.loggers.get('main');
    var models = app.get('models');

    logger.debug("loadItemsParValue on " + file);
    logger.debug( "ExpectedTimeout : " + expectedExecutionTime);

    // Read the file here !
    // Note ! The file is loaded in full here and is available in the data variable !
    // It is possible to have to limit the file dimension !
    fs.readFile(file, 'utf8', function (err, data) {
        if (err) {
            logger.error("Reading file error : " + err.toString());
            // What to do ???
        } else {
            // The reading/parsing the file data is done in a separate thread thus the promise.
            loadingPromise = readItemsParValue(data);

            loadingPromise.then(function (result) {
                logger.info("Parsed data successfull !");
                // FRES-176
                // Note - here was called the tbl_sap_item_par_value update in order to fill up
                // the material description and the vendor name. Removed from here because it was slowing down
                // the upload process, causing a crash.  The update MUST be done manually after the download,
                // runnnig a query directly on the DB Or editing any item from the web console
                awcblcommon.archiveFile(app, file, data);
                loadingPromise = null;      // Reset load promise
            }, function (err) {
                logger.error("Parsing data errors ! Aborted ! " + err);
                loadingPromise   = null;
            });
        }
    });

    //  TEMP ! To quick switch between the two things
    var usePromise = false;

    // Load promise callback function
    // Parsing data from item par value file
    // Generic :
    //   - load clinic profile table
    //   - load master item table
    //   - parse file data read - for each record
    //     - verify if some data are ok (clinic number/master item)
    // Using promise :
    //     - create promise to save in the db - increase promise counter
    //     - decrease promise counter when promise executed
    // No using promise
    //     - save record in db
    function readItemsParValue (data) {
        return new Promise(function (resolve, reject) {
            var allMasterItems = [];        // List of master items records
            var allValidClinics = [];       // List of clinic records

            var lines = data.split(/\r?\n/);
            var lineCounter = 0;
            var id, id1 = -1;
            var lineProcessedCounter = 0;
            var canProcess = true;

            var inClinicId;                 // Input clinic ID
            var inProductId;                // Input product ID
            var inVendorId;                 // Input vendor ID
            var inQoh;                      // Input QOH value
            var inMinParValue;              // Input Min par value
            var inMaxParValue;              // Input Max par value

            logger.info("readItemsParValue : " + data);

            // CAUTION !!
            // Probably all this part can be removed since the input file is supposed to come already validated !

            //allValidClinics = awcblcommon.loadClinicProfile(app, debug);
            //allMasterItems  = awcblcommon.loadMasterItems(app, debug);

            models.ClinicProfile.findAll().then(function (clinics) {
                if (clinics) {
                    if (debug === true) {
                        logger.debug("loaded " + clinics.length + " Clinic Profiles");
                    }

                    allValidClinics = clinics;

                    models.MasterItem.findAll().then(function (items) {
                        if (items) {
                            if (debug === true) {
                                logger.debug("loaded " + items.length + " Master Items");
                            }
                            allMasterItems = items;

                            if ((allValidClinics.length > 1) && (allMasterItems.length > 1)) {
                                if (debug === true) {
                                    logger.debug("Start processing " + lines.length + " read from file");
                                }
                            } else {
                                logger.debug("WTF ? " );
                                reject("Something wrong with the loaded master_item or clinic tables");
                            }

                            // List for validation loaded - proceed

                            //
                            // ---------------- MAIN LOOP FOR READING FILE ---------------------------------------
                            //
                            // lineCounter is used to read the content line by line
                            // lineProcessedCounter is used to process a fixed number of lines so to limit how many lines are processed in a fixed amount of time
                            // promiseCounter is used to verify that every promise created is actually fullfilled
                            lineCounter = 0;
                            lineProcessedCounter = 0;
                            processedCounter = 0;
                            canProcess = true;

                            while (lineCounter < lines.length) {
                                // Read record to be saved
                                var csv_line = lines[lineCounter].split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);
                                csv_line = csv_line || [];

                                if (csv_line.length > 1) {
                                    inClinicId = csv_line[0].split(',').join('');   // Assign input Clinic ID
                                    inProductId = csv_line[1].split(',').join('');   // Assign input Product ID
                                    inQoh = csv_line[2].split(',').join('');   // Assign input QOH
                                    inMinParValue = csv_line[3].split(',').join('');   // Assign input MinParValue
                                    inMaxParValue = csv_line[4].split(',').join('');   // Assign input MaxParValue

                                    // Always display basic read data
                                    if (debug === true) {
                                        logger.debug("-------------- line " + lineCounter + " -------------------");
                                        logger.debug("line read" + csv_line);
                                        logger.debug("     clinicID : " + inClinicId);  // Clinic ID
                                        logger.debug("    productId : " + inProductId);  // Product ID
                                    }

                                    // Verify if the clinic_id from the line exists
                                    if (inClinicId) {
                                        id = awcblcommon.getClinicId(allValidClinics, inClinicId.trim());
                                        if (id < 0) {
                                            logger.error("Clinic ID for [" + inClinicId.trim() + "] - NOT FOUND ! Ignore !");
                                            continue;
                                        }
                                    }

                                    // Verify if the product from the line exists
                                    if (inProductId) {
                                        id1 = awcblcommon.getProductId(allMasterItems, inProductId.trim());
                                        if (id1 < 0) {
                                            logger.error("Product ID for [" + inProductId.trim() + "] - NOT FOUND ! Ignore !");
                                            continue;
                                        }
                                    }

                                    // Here the clinic ID and product ID are existing - ready to proceed
                                    // Prepare correct ID for the record and verify that it doesn't exists in the
                                    // tbl_sap_item_par_value table.
                                    // If not, add it
                                    // If it does exists - update it with the new values

                                    inVendorId = awcblcommon.getProductVendorId(allMasterItems, id1);

                                    if (debug === true) {
                                        logger.debug("     vendorId : " + inVendorId);  // Vendor ID
                                        logger.debug("          QOH : " + inQoh);  // QOH
                                        logger.debug("min Par value : " + inMinParValue);  // Min
                                        logger.debug("max Par value : " + inMaxParValue);  // Max
                                    }

                                    // Counters management
                                    lineProcessedCounter++;
                                    processedCounter++;
                                    logger.debug("   increased processed counter : " + processedCounter);

                                    if (usePromise) {
                                        if (debug === true) {
                                            logger.debug("---> save product : " + lineProcessedCounter);     // TEMP
                                        }

                                        loadingRecPromise = promiseSaveItemSapParValue(inProductId.trim(),
                                          inVendorId.trim(),
                                          inClinicId.trim(),
                                          inQoh,
                                          inMinParValue,
                                          inMaxParValue,
                                          debug);

                                        // Callback promise top save data
                                        loadingRecPromise.then(response => {
                                            lineProcessedCounter--;

                                            if (debug === true) {
                                                logger.debug("<--- saved product : " + lineProcessedCounter);
                                            }
                                            //loadingRecPromise = null;

                                            // Issue the resolve for the loadingPVPromise only when all the
                                            // created loadingRecPromise are done (i.e. the table is populated
                                            //                        if (lineProcessedCounter === 0 && promiseCounter === 0) {
                                            //                            resolve(1);
                                            //                        }
                                        }, err => {
                                            // Error trying to store a valid data - abort operation and notify !
                                            loadingRecPromise = null;
                                            logger.error(err);
                                            reject(err);
                                        });

                                    } else {
                                        if (plainSaveItemSapParValue(inProductId.trim(),
                                                                  inVendorId.trim(),
                                                                  inClinicId.trim(),
                                                                  inQoh,
                                                                  inMinParValue,
                                                                  inMaxParValue,
                                                                  lineProcessedCounter,
                                                                  debug) === false) {
                                            // Abort !!!
                                            logger.error("Stored records failed : " + lineProcessedCounter);
                                            break;
                                        }

                                    }
                                }
                                lineCounter++;
                            } // end loop

                            // The idea here is to wait for the processedCounter to go back to zero before
                            // to continue
//                            while (processedCounter > 0) {
//                                logger.info(".");
//                                sleep(100);
//                            }

                            logger.info("End file loading in DB");
                            resolve(1);

                        } else {
                            if (debug === true) {
                                logger.error("No Master Items loaded");
                            }
                            reject("No Master Items loaded");
                        }
                    });
                } else {
                    if (debug === true) {
                        logger.error("No Clinic Profiles loaded");
                        reject("No clinic Profiles loaded");
                    }
                }
            });
        });
    }

    function sleep(milliseconds) {
        const date = Date.now();
        let currentDate = null;
        do {
            currentDate = Date.now();
        } while (currentDate - date < milliseconds);
    }

    // Promise callback function to save a CSV line in DB
    // CAUTION !!!  This promise has set a timeout !
    // Be sure the saving operations are performed BEFORE the timeout expires !!
    function promiseSaveItemSapParValue (vendor_material, vendor_id, clinic_id, qoh, min, max, debug) {
        return new Promise(function (resolve, reject) {

            // Set up the timeout - if the loading operations are not done in the expectedExecutionTime , the timeout
            // will force to return error.
            var errTimeout = setTimeout(function () {
                reject('Promise timed out after ' + expectedExecutionTime + ' ms');
            }, expectedExecutionTime);

            if (vendor_material && vendor_id && clinic_id) {
                models.VendorSapParValue.findOrCreate({
                    where: {
                        vendorMaterial: vendor_material,
                        vendorId: vendor_id,
                        clinicId: clinic_id
                    }, defaults: {
                        qohValue: qoh,
                        minParValue: min,
                        maxParValue: max
                    }
                }).spread(function (data, created) {
                    if (!created) {
                        data.vendorMaterial = vendor_material;
                        data.vendorId = vendor_id;
                        data.clinicId = clinic_id;
                        data.qohValue = qoh;
                        data.minParValue = min;
                        data.maxParValue = max;

                        data.save().then(function () {
                            if (debug === true) {
                                logger.debug("   updated !");
                            }
                        }).catch(function (error) {
                            logger.error(error);
                        });
                    } else {
                        if (debug === true) {
                            logger.debug("   created !");
                        }
                    }
                    processedCounter--;
                    clearTimeout(errTimeout);       // Cancel timeout
                    logger.debug("   decreased promise counter : " + processedCounter);
                    resolve(1);

                }).catch(function (err) {
                    clearTimeout(errTimeout);       // Cancel timeout
                    logger.error(err);
                    reject(err);
                });
            } else {
                clearTimeout(errTimeout);       // Cancel timeout
                reject("No valid input");
            }
        });
    }

    // Save a CSV line in DB
    // It seems the sequelizer creates a thread to be execute later
    function plainSaveItemSapParValue (vendor_material, vendor_id, clinic_id, qoh, min, max, proc_counter, debug) {
        // Set up the timeout - if the loading operations are not done in the expectedExecutionTime , the timeout
        // will force to return error.
        if (vendor_material && vendor_id && clinic_id) {
            models.VendorSapParValue.findOrCreate({
                where: {
                    vendorMaterial: vendor_material,
                    vendorId: vendor_id,
                    clinicId: clinic_id
                }, defaults: {
                    qohValue: qoh,
                    minParValue: min,
                    maxParValue: max
                }
            }).spread(function (data, created) {
                if (!created) {
                    data.vendorMaterial = vendor_material;
                    data.vendorId = vendor_id;
                    data.clinicId = clinic_id;
                    data.qohValue = qoh;
                    data.minParValue = min;
                    data.maxParValue = max;

                    data.save().then(function () {
                        if (debug === true) {
                            logger.debug("   updated " + proc_counter + " ! (processedCounter : " + processedCounter);
                        }
                    }).catch(function (error) {
                        logger.error(error);
                        return false;
                    });
                } else {
                    if (debug === true) {
                        logger.debug("   created " + proc_counter + " ! (processedCounter : \" + processedCounter");
                    }
                }
                processedCounter--;
                return true;
            });
        } else {
            return false;
        }
    }
}
