var path = require('path');
var fs = require('fs');

module.exports = function(winston, config) {
    var logDir = config.log.logDir;

    if (logDir === "") logDir = path.join(__dirname, "../..", "/log");

    if (logDir.startsWith(".")) logDir = path.join(__dirname, logDir);

    var myCustomLevels = {
        levels: { error: 0, warn: 1, info: 2, verbose: 3, debug: 4, trace: 5 },
        colors: {
            trace: 'blue',
            debug: 'green',
            verbose: 'green',
            info: 'green',
            warn: 'yellow',
            error: 'red'
        }
    };


    //winston.setLevels(myCustomLevels.levels);
    winston.level = 'debug';

    fs.exists(logDir, function(exists) {
        if (!exists) {
            fs.mkdir(logDir);
        }
    });

    var txnOpts = {
        filename: logDir + '/m3-api.error.log',
        level: 'error',
        name: "m3-api.error"
    };

    var defaultLevel = config.log.defaultLevel || "debug";

    var mainOpts =  {
        filename: logDir + '/m3-api.log',
        name: "m3-api",
        level: defaultLevel
    };

    var logger = new (winston.Logger)({
        transports: [
            new winston.transports.File(mainOpts),
            new winston.transports.File(txnOpts),
            new winston.transports.Console()
        ]
    });

    function Win() {
        this.main = logger;
        this.transaction = logger;
        this.loggers = this;

    }

    Win.prototype.get = function(key) {
        return this[key];
    };

    return new Win();
};