USE m3console;
-- BACKUPS
CREATE TABLE IF NOT EXISTS tmp_item_catalog_0201 AS SELECT * FROM `tbl_item_catalog`;
CREATE TABLE IF NOT EXISTS tmp_clinic_catalog_0201 AS SELECT * FROM `tbl_clinic_catalog`;
CREATE TABLE IF NOT EXISTS tmp_vendor_list_0201 AS SELECT * FROM `tbl_vendor_list`;


RENAME TABLE `tbl_vendor_list` to `tbl_catalogs`;
ALTER TABLE `tbl_catalogs` CHANGE `vendor_id` `catalog_id` VARCHAR(255) NOT NULL COLLATE 'utf8_general_ci';
ALTER TABLE `tbl_catalogs` CHANGE `vendor_display` `catalog_display` VARCHAR(255) NOT NULL COLLATE 'utf8_general_ci';

ALTER TABLE `tbl_item_catalog` CHANGE `vendor_id` `catalog_id` INT(11) NOT NULL;
ALTER TABLE `tbl_clinic_catalog` CHANGE `vendor_id` `catalog_id` INT(11) NOT NULL;

CREATE TABLE `tbl_vendor_list` (
  `id` INT(11) NOT NULL AUTO_INCREMENT,
  `vendor_id` VARCHAR(255) NOT NULL COLLATE 'utf8_general_ci',
  `vendor_display` VARCHAR(255) NOT NULL COLLATE 'utf8_general_ci',
  `minimum_order_amount` DECIMAL(9,2) DEFAULT 0.00,
  `fee` DECIMAL(9,2) DEFAULT 0.00,
  `date_modified` TIMESTAMP(3) NULL DEFAULT NULL,
  PRIMARY KEY (`id`)
)
  COLLATE='latin1_swedish_ci'
  ENGINE=InnoDB
  AUTO_INCREMENT=1
;

-- FRES-112
-- Create tables to be used for the new app functionality (SAP vs IO files - par_value added to SAP file)

-- Create table tbl_sap_item_par_value
--
CREATE TABLE `tbl_sap_item_par_value` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `vendor_material` varchar(255) CHARACTER SET utf8 NOT NULL DEFAULT '',
  `vendor_id` varchar(255) CHARACTER SET utf8 NOT NULL DEFAULT '',
  `clinic_id` varchar(255) CHARACTER SET utf8 NOT NULL DEFAULT '',
  `material_description` varchar(255) CHARACTER SET utf8 DEFAULT NULL,
  `min_par_value` int(11) NOT NULL DEFAULT '0',
  `max_par_value` int(11) DEFAULT '0',
  `qoh_value` int(11) DEFAULT '0',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1030 DEFAULT CHARSET=latin1;

-- Create table tbl_vendor_route
--
CREATE TABLE `tbl_vendor_route` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `vendor_id` varchar(255) CHARACTER SET utf8 NOT NULL,
  `vendor_display` varchar(255) CHARACTER SET utf8 NOT NULL,
  `vendor_route` varchar(255) CHARACTER SET utf8 NOT NULL DEFAULT 'IO',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=7 DEFAULT CHARSET=latin1;

-- Create table tbl_vendor_route_product
--
CREATE TABLE `tbl_vendor_route_product` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `sap_material_id` int(11) NOT NULL,
  `vendor_material` varchar(255) CHARACTER SET utf8 NOT NULL DEFAULT '',
  `vendor_id` varchar(255) CHARACTER SET utf8 DEFAULT '',
  `vendor_route` varchar(255) CHARACTER SET utf8 DEFAULT '',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1024 DEFAULT CHARSET=latin1;

-- Create table tbl_vendor_route_app
-- This table is sent to the terminal
-- The content of the table is the tbl_vendor_route + tbl_vendor_route_clinic
-- that can overwrite the tbl_vendor_route content  (see FRES-169)
CREATE TABLE tbl_vendor_route_app (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `vendor_id` varchar(255) CHARACTER SET utf8 DEFAULT '',
  `vendor_route` varchar(255) CHARACTER SET utf8 DEFAULT '',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1024 DEFAULT CHARSET=latin1;

-- FRES-149 - VOR clinic oriented
-- Table creation
-- A new table is needed to handle the new VOR clinic oriented functionality keeping compatibiluty
-- with older applications
CREATE TABLE `tbl_vendor_route_clinic` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `clinic_id` varchar(255) CHARACTER SET utf8 NOT NULL DEFAULT '',
  `vendor_id` varchar(255) CHARACTER SET utf8 NOT NULL,
  `vendor_display` varchar(255) CHARACTER SET utf8 NOT NULL,
  `vendor_route` varchar(255) CHARACTER SET utf8 NOT NULL DEFAULT 'IO',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=17 DEFAULT CHARSET=latin1;


-- Populate tbl_vendor_list
INSERT INTO `tbl_vendor_list` (vendor_id, vendor_display) SELECT catalog_id, catalog_display FROM `tbl_catalogs` WHERE is_med_group = 1;

-- populate tbl_vendor_route - this is supposed to be done once manually
-- The queries will leave the tbl_vendor_route populated with the vendors presents in the tbl_vendor_list, forcing the route to IO
-- Subsequent edit of the route will be needed in order to assign the correct routing for vendor
--
delete from `tbl_vendor_route`;
alter table `tbl_vendor_route` auto_increment=1;
insert into `tbl_vendor_route` (id, vendor_id, vendor_display, vendor_route)
select id, vendor_id, vendor_display, "IO" from `tbl_vendor_list`;

-- populate tbl_vendor_route_clinic - this is done every time a field is added/edited to the table
-- Filling up vendor description in tbl_vendor_route_clinic from tbl_vendor_list
update `tbl_vendor_route_clinic`
set  vendor_display = if (exists
(select vendor_display from tbl_vendor_list where tbl_vendor_route_clinic.vendor_id = vendor_id),
(select vendor_display from tbl_vendor_list where tbl_vendor_route_clinic.vendor_id = vendor_id), "Invalid vendor");

-- populate tbl_vendor_route_app for old app version - is called every time an old app request
-- the vendor product route table
-- The queries will leave the tbl_vendor_route_app filled up from valid products from tbl_master_items
-- and the routing from the tbl_vendor_route table.
-- It is implied the tbl_vendor_route table to be correctly filled up and in sync with the tbl_vendor_list table before
-- to execute this queries
-- Modified because FRES-169
--
delete from tbl_vendor_route_app;
alter table tbl_vendor_route_app auto_increment=1;
insert into tbl_vendor_route_app (vendor_id, vendor_route)
	select vendor_id, (select vendor_route from tbl_vendor_route where vendor_id = tbl_master_item.vendor_id)
	;

-- populate tbl_vendor_route_app - this is executed inside the code every time an application (>= 1.6.15)
-- request the vendor route product table
-- The queries will leave the tbl_vendor_route_app filled up from the tbl_vendor_route and tbl_vendor_route_clinic table.
-- It is implied the tbl_vendor_route table to be correctly filled up and in sync with the tbl_vendor_list table before
-- to execute this queries
-- The fixed item ".9999BM" in the code is replaced with the one coming from the app.
-- Modified for FRES-169
--
delete from `tbl_vendor_route_app`;
alter table tbl_vendor_route_app auto_increment=1;
insert into tbl_vendor_route_app (vendor_id, vendor_route)
	select vendor_id if (exists (select vendor_route from tbl_vendor_route_clinic where
	(vendor_id=tbl_master_item.vendor_id and clinic_id=".9999BM")),
	(select vendor_route from tbl_vendor_route_clinic where (vendor_id=tbl_master_item.vendor_id and clinic_id=".9999BM")),
	(select vendor_route from tbl_vendor_route where vendor_id = tbl_master_item.vendor_id))
	;


-- FRES-128 - activity logging
-- Table creation
CREATE TABLE `tbl_activity_log` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `user_id` varchar(50) CHARACTER SET utf8 NOT NULL,
  `name_table` varchar(255) CHARACTER SET utf8 DEFAULT NULL,
  `description` varchar(2500) CHARACTER SET utf8 DEFAULT NULL,
  `date_modification` timestamp(3) NULL DEFAULT NULL,
  PRIMARY KEY (`id`),
  UNIQUE KEY `idx_tbl_ftp_device_user_stat_uq` (`user_id`,`date_modification`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;


/*
CREATE PROCEDURE prc_convert_catalog_ids()
  BEGIN
    DECLARE done INT DEFAULT FALSE;
    DECLARE vendor_cursor CURSOR FOR SELECT id, vendor_id FROM tbl_vendor_list;
    DECLARE vendor_id VARCHAR(255) COLLATE 'utf8_general_ci';
    DECLARE vid INT(11);
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

    OPEN vendor_cursor;

    read_loop: LOOP
      FETCH vendor_cursor INTO vid, vendor_id;
      IF done THEN
        LEAVE read_loop;
      END IF;
      UPDATE `tbl_catalogs` SET id = vid WHERE catalog_id = vendor_id;
    END LOOP;

    CLOSE vendor_cursor;

  END;

CALL prc_convert_catalog_ids();


ALTER TABLE `tbl_item_catalog` DROP FOREIGN KEY `FK_Item_catalog_Vendor`;
ALTER TABLE `tbl_item_catalog` ADD CONSTRAINT `FK_Item_catalog_Vendor` FOREIGN KEY (`catalog_id`) REFERENCES `tbl_catalogs` (`id`) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE `tbl_clinic_catalog` DROP FOREIGN KEY `FK_Clinic_catalog_Vendor`;
ALTER TABLE `tbl_clinic_catalog` ADD CONSTRAINT `FK_Clinic_catalog_Vendor` FOREIGN KEY (`catalog_id`) REFERENCES `tbl_catalogs` (`id`) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE `tbl_vendor_list` DROP COLUMN is_med_group;
ALTER TABLE `tbl_vendor_list` ADD (minimum_order_amount DECIMAL(9,2) DEFAULT 0.00);
 */

ALTER TABLE `tbl_master_item` ADD (vendor_id VARCHAR(255) COLLATE 'utf8_general_ci');
ALTER TABLE `tbl_master_item` ADD (unit_price DECIMAL(9,2) DEFAULT 0.00);

DELETE FROM `tbl_catalogs` WHERE is_med_group = 1;
ALTER TABLE `tbl_catalogs` DROP COLUMN is_med_group;