/**
 * Created by Ali on 7/21/2020 .
 *
 * Common routines for handling RabbitMQ queues
 *
 */
function RabbitHelper(winston, config) {
  this.rabbit = require("rabbot");
  var logger = winston.loggers.get('main');

  require("../config/RabbitTopology.js")(this.rabbit, null, config)
    .then(function() {
     // logger.debug("Rabbit Topology Loaded - config : " + JSON.stringify(config, null, 4));
     logger.info("Rabbit Topology Loaded");
    });

  this.rabbit.on("unreachable", function() {
    logger.info("RABBIT Server Unreachable - configuration used : ");
    logger.info("       host -> " + config.rabbitConfig.host);
    logger.info("       user -> " + config.rabbitConfig.user);
    logger.info("   password -> " + config.rabbitConfig.pass);
    logger.info("       port -> " + config.rabbitConfig.port);
    logger.info("      vhost -> " + config.rabbitConfig.vhost);
    process.exit();
  });
}

RabbitHelper.prototype.publishLoader = function(winston, msg, fileType) {
  var self = this;
  var direction = fileType;
  var logger = winston.loggers.get('main');

  self.rabbit.publish("topic-fresenius-loader", {
    routingKey: direction,
    type: direction,
    body: msg,
    persistent: true
  }).then(function() {
    logger.info("published message: " + msg);
  }).catch(function(err) {
    logger.info("Did not publish message" + err);
  });
};

// Process excel file
RabbitHelper.prototype.processFile = function(winston, file, fileType) {
   var i;
  var fs = require('fs');
  const csv = require('csv-parser');
  var logger = winston.loggers.get('main');
  var self = this;

  fs.createReadStream(file).pipe(csv({
      headers: false
    }))
    .on('data', function (row) {
      var message = "";
      for (i = 0; i < Object.keys(row).length; i++) {
        message += row[i] + ",";
      }
      message = message.slice(0, -1);
      self.publishLoader(winston, message, fileType);
    }).on('end', function () {
      logger.info("Completed messaging");
  });
};

// Process Data
RabbitHelper.prototype.processData = function(winston, data, fileType) {
  var i;
  var fs = require('fs');
  const csv = require('csv-parser');
  var logger = winston.loggers.get('main');
  var self = this;

  var message = "";
  for (i = 1; i < data.length; i++) {
    message = JSON.parse(JSON.stringify(data[0])) + "," + JSON.parse(JSON.stringify(data[i]));
    self.publishLoader(winston, message, fileType);
  }
};



RabbitHelper.prototype.cleanUp = function(cb) {
  this.rabbit.shutdown().then(function() {
    if (cb) {
      cb(null, null);
    }
  }).catch(function(err) {
    if (cb) {
      cb(err, null);
    } else {
      throw err;
    }
  });
};

module.exports = RabbitHelper;