angular.module('app').controller('AsnCtrl', [
    '$scope',
    '$http',
    '$animate',
    'Constants',
    'FileSaver',
    'Blob',
    '$timeout',
    '$interval',
    function($scope, $http, $animate, Constants, FileSaver, Blob, $timeout, $interval) {

        $scope.data = [];

        $scope.oldLimit = 50;
        $scope.oldOffset = 0;

        //initializing busy indicator for export
        $scope.busy = {
            message: 'Exporting Data...',
            backdrop: true,
            busyPromise: null
        };

        //initializing date picker
        $scope.dateDbA = moment();
        $scope.dateDbB = moment();

        $scope.optionsDbA = {
            callback: fromUpdated
        };
        $scope.optionsDbB = {
            callback: toUpdated
        };
        $scope.menuOpened = false;

        //view methods
        $scope.exportData = function() {
            var request = requestData;
            $scope.oldLimit = request.limit;
            $scope.oldOffset = request.offset;

            delete request.limit;
            delete request.offset;

            $scope.busy.busyPromise = getAsnFilePromise();

        };

        $scope.toggleMenu = function(event) {
            $scope.menuOpened = !($scope.menuOpened);
            event.stopPropagation();
        };

        $scope.closeThis = function() {
            if ($scope.menuOpened) {
                $scope.menuOpened = false;
            }
        };
        var start = new Date();
        var sec = $interval(function () {
            var wait = parseInt(((new Date()) - start) / 1000, 10);
            $scope.wait = wait + 's';
        }, 1000);
        function rowTemplate() {
            return $timeout(function() {
                $scope.waiting = 'Done!';
                $interval.cancel(sec);
                $scope.wait = '';
                return '<div ng-class="{ \'red\': grid.appScope.rowFormatter( row ) }">' +
                    '  <div ng-repeat="(colRenderIndex, col) in colContainer.renderedColumns track by col.colDef.name" class="ui-grid-cell" ng-class="{ \'ui-grid-row-header-cell\': col.isRowHeader }"  ui-grid-cell></div>' +
                    '</div>';
            }, 1000);
        }

        $scope.rowFormatter = function( row ) {
            var dt = (moment(row.entity.asnDate).add(1, 'days') < moment());
            if (dt) {
                dt = dt && ((row.entity.receivedQty + row.entity.lostQty) !== row.entity.shipQty);
            }
            return dt;
        };

        //initialization for grid and column headers
        $scope.gridOptions = Constants.getDefaultGridOptions();
        $scope.gridOptions.useExternalSorting = true;
        $scope.gridOptions.useExternalPagination = true;
        $scope.gridOptions.useExternalFiltering = true;
        $scope.gridOptions.flatEntityAccess = true;
        $scope.gridOptions.rowTemplate= rowTemplate();
        $scope.gridOptions.columnDefs = Constants.asnCols;

        $scope.gridOptions.onRegisterApi =  function( gridApi ) {
            $scope.gridApi = gridApi;
            $scope.gridApi.core.on.sortChanged($scope, function(grid, sortColumns) {
                onSortChanged(sortColumns);
            });
            $scope.gridApi.pagination.on.paginationChanged($scope, function(newPage, newSize) {
                requestData.offset = (newPage - 1) * newSize;
                requestData.limit = newSize;
                getPage();
            });
            $scope.gridApi.core.on.filterChanged($scope, onFilterChanged)
        };

        var requestData = {
            offset: 0,
            limit: 50,
            order: "asnDate DESC"
        };

        getPage();

        function getPage() {
            $http.post('/api/getAsnData/', requestData, {timeout: 60000})
                .then(function(res) {
                    console.log("getpagedone");
                    $scope.gridOptions.data = res.data;
                    $scope.gridOptions.totalItems = res.headers('x-total-count');
                });
        }

        function getAsnFilePromise() {
            return $http.post('/api/v1/getAsnFile/', requestData, {timeout: 60000}).then(function(res) {
                var file = new Blob([ res.data ], {
                    type : 'text/plain;charset=utf-8a'
                });
                FileSaver.saveAs(file, 'asns.csv');

                requestData.limit = $scope.oldLimit;
                requestData.offset = $scope.oldOffset;

            });
        }

        function onSortChanged(sortColumns) {
            if (sortColumns.length != 0) {
                requestData.order = "'" + sortColumns[0].name + "' " + sortColumns[0].sort.direction;
                getPage();
            }

        }

        function onFilterChanged() {
            var grid = $scope.gridApi.grid;

            for (var i = 0; i < grid.columns.length; i++) {
                var column = grid.columns[i];
                if (column.filters[0].term != null && column.filters[0].term != "") {
                    if (requestData["where"] == null) {
                        requestData["where"] = {};
                    }
                    requestData["where"][column.field] = column.filters[0].term;

                } else {
                    if (requestData["where"] != null && requestData["where"][column.field] != null) {
                        delete requestData["where"][column.field];
                    }
                }
            }
            getPage();
        }

        $scope.forceFilter = function() {
            return onFilterChanged();
        }

        function fromUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.asnDate == null) {
                requestData.where.asnDate = {};
            }

            requestData.where.asnDate['$gt'] = date.startOf('day');
            $scope.fromDate = date.format("MM-DD-YYYY");
            getPage();
        }

        function toUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.asnDate == null) {
                requestData.where.asnDate = {};
            }

            requestData.where.asnDate['$lt'] = date.endOf('day');
            $scope.toDate = date.format("MM-DD-YYYY");
            getPage();
        }
    }
]);