/**
 * Created by will on 10/2/15.
 * Modified by TFG on 10/20/2019.
 * Modified by TFG on 11/19/2019.
 */
angular.module('app').controller('MasterItemCtrl', [
    '$scope',
    '$window',
    '$http',
    'uiGridConstants',
    'Constants',
    'ngDialog',
    '$timeout',
    'FileSaver',
    'Blob',

    function($scope, $window, $http, uiGridConstants, Constants, ngDialog, $timeout, FileSaver, Blob) {
        var requestData = {};
        var existingRow = {};
        var recordEdited = false;   // FRES-128 - flag to discrimintate between adding and editing

        $scope.loading = {
            message: 'Loading Data...',
            backdrop: true,
            busyPromise: null
        };

        $scope.savingData = {
            message: 'Saving Data...',
            backdrop: true,
            busyPromise: null
        };

        $scope.busy = {
            message: 'Please wait...',
            backdrop: true,
            busyPromise: null
        };

        $scope.addActivityLog = {
            message: 'Add activity log...',
            backdrop: true,
            busyPromise: null
        };

        $scope.gridOptions = Constants.getDefaultGridOptions();
        $scope.gridOptions.columnDefs = Constants.masterItemCols;
        $scope.gridOptions.onRegisterApi = function(gridApi) {
            $scope.gridApi = gridApi;
            $scope.gridApi.core.on.filterChanged($scope, onFilterChanged);
        };
        $scope.gridOptions.totalItems = 0;

        function getPage() {
            return $http.post('/api/v2/getMasterItem', requestData).then(function(res){
                $scope.gridOptions.data = null;
                $scope.gridOptions.data = res.data;
            },function(err) {
                $scope.gridOptions.data = null;
                console.log("NO REFRESH!!!!" + err);
            });
        }

        function reloadPage() {
            //need a delay for catalog inserts to commit
            $timeout(function() {
                $scope.loading.busyPromise = getPage();
            }, 300);
        }

        $scope.isDisabled = false;
        $scope.showSapVerifyAlert = false;
        $scope.showVendorVerifyAlert = false;

        $scope.loading.busyPromise = getPage();

        $scope.digits = {};

        //initializing date picker
        $scope.dateDbA = moment();
        $scope.dateDbB = moment();

        $scope.optionsDbA = {
            callback: fromUpdated
        };
        $scope.optionsDbB = {
            callback: toUpdated
        };
        $scope.menuOpened = false;

        $scope.toggleMenu = function(event) {
            $scope.menuOpened = !($scope.menuOpened);
            event.stopPropagation();
        };

        $scope.closeThis = function() {
            if ($scope.menuOpened) {
                $scope.menuOpened = false;
            }
        };

        function openEditDialog(row) {
            //Using editCatalog as a bindable Object in the modal.... cloning the original Catalog list
            $scope.editCatalog = angular.copy($scope.catalogList);
            //Mark a new "Checked" field, based on vendorList string
            if (row != null) {
                //edit mode
                for (var v = 0; v < $scope.editCatalog.length; v++) {
                    $scope.editCatalog[v].checked = $scope.isCheckboxChecked($scope.editCatalog[v].catalogId);
                }
            } else {
                //add mode
                for (var v = 0; v < $scope.editCatalog.length; v++) {
                    $scope.editCatalog[v].checked = $scope.editCatalog[v].isDefault;
                }
            }
            //$scope.editCatalog = angular.copy($scope.catalogList);
            ngDialog.open({
                template: '/masterItem/masterItemMaintenance.html',
                showClose: true,
                className: "ngdialog-theme-default masterItem",
                scope: $scope
            });
        }

        function getCatalogs(row) {
            if ($scope.catalogList == null) {
                console.log("getting Catalogs");
                return $http.post('/api/getCatalogList', null).then(function (res) {
                    $scope.catalogList = res.data;
                    getVendors(row);
                }, function (err) {
                    $scope.catalogList = null;
                });
            } else {
                getVendors(row);
            }
        }

        function getVendors(row) {
            if ($scope.vendorList == null) {
                console.log("getting Vendors");
                return $http.post('/api/getVendorList', null).then(function (res) {
                    $scope.vendorList = res.data;
                    openEditDialog(row);
                }, function (err) {
                    $scope.vendorList = null;
                });
            } else {
                openEditDialog(row);
            }
        }

        $scope.formatPrice = formatPrice;

        function formatPrice (csvPrice) {
            if (!csvPrice) return csvPrice;
            var flPrice;
            try {
                flPrice = parseFloat(csvPrice);
                return flPrice.toFixed(2);
            } catch (ex) {
                return "0.00";
            }
        }

        $scope.addRowModal = function (row) {
            $scope.editRow = { cntInd: false, countType: "", medVendor: "" };
            if (row != null) {
                $scope.editRow = findById($scope.gridOptions.data, row.entity.id);
                $scope.editRow.unitPrice = formatPrice($scope.editRow.unitPrice);
                if (JSON.stringify($scope.editRow.medVendor) == "null") $scope.editRow.medVendor = "";
            }
            getCatalogs(row);
        };

        $scope.deleteRow = function(row) {
            if (row != null) {
                var doDelete = $window.confirm("Are you sure?");
                if (doDelete) {
                    $scope.loading.busyPromise = $http.post('/api/deleteMasterItem', row.entity).then(function(res) {
                        // FRES-128 - add entry in the activity log
                        $scope.addActivityLog.busyPromise = addActivityLogEntry('tbl_master_item', 'removed record [' +
                            row.entity.sapMaterialId + " + " + row.entity.vendorMaterial + "]");

                        getPage();
                    });
                }
            }
        };

        $scope.saveMasterItem = function(row, vendors) {
            $scope.editRow.dateModified = new Date();
            //Before saving, build "new" vendorList string
            $scope.editRow.medVendor = "";
            var firstEntry = true;
            for (var v = 0; v < vendors.length; v++ ) {
                if (vendors[v].checked) {
                    if (!firstEntry) {
                        $scope.editRow.medVendor += ",";
                    }
                    $scope.editRow.medVendor += vendors[v].id;
                    firstEntry = false;
                }
            }

            $scope.savingData.busyPromise = $http.post('/api/v2/saveMasterItem', $scope.editRow)
            .then(function(res) {
                // FRES-128 - add entry in the activity log
                var msg = recordEdited === false ? "added record [" : "modified record [";
                $scope.addActivityLog.busyPromise = addActivityLogEntry('tbl_master_item', msg +
                    $scope.editRow.sapMaterialId + " + " + $scope.editRow.vendorMaterial + "]");
                recordEdited = false;       // Reset flag add/edit

                $scope.closeDialog();
                reloadPage();

            }, function(err) {
                console.log("error: " + err);
                recordEdited = false;       // Reset flag add/edit
                $scope.closeDialog();
                reloadPage();
            });
        };

        $scope.verifyMasterItem = function() {
            console.log("verifying");
            recordEdited = false;       // Reset flag add/edit

            var row = {};
            row.vendorMaterial = $scope.editRow.vendorMaterial;
            row.sapMaterial = $scope.editRow.sapMaterial;
            //query for the combo
            $http.post('/api/v2/getMasterItem', row)
                .then(function(res) {
                    if (res.data[0] != null) {
                        existingRow = res.data[0];
                        if (existingRow.sapMaterial != null && existingRow.sapMaterial == row.sapMaterial) {
                            $scope.showSapVerifyAlert = true;
                            recordEdited = true;
                        } else if (existingRow.vendorMaterial != null && existingRow.vendorMaterial == row.vendorMaterial) {
                            $scope.showVendorVerifyAlert = true;
                            recordEdited = true;
                        }
                    } else {
                        $scope.isDisabled = true;
                    }
                }, function(err) {
                    console.log("Error checking Master Item: " + err);
                    $scope.closeDialog();
                })
        };

        $scope.editExistingRow = function() {
            $scope.editRow = existingRow;
            $scope.showSapVerifyAlert = false;
            $scope.showVendorVerifyAlert = false;
            $scope.isDisabled = true;
        };

        $scope.closeDialog = function(){
            ngDialog.close();
            $scope.showSapVerifyAlert = false;
            $scope.showVendorVerifyAlert = false;
            $scope.isDisabled = false;
        }

        $scope.exportData = function() {
            var request = requestData;
            //delete request.limit;
            //delete request.offset;

            $scope.busy.busyPromise = getMasterItemPromise();
        };

        $scope.messageData = function() {
            var request = requestData;
            //delete request.limit;
            //delete request.offset;

            $scope.busy.busyPromise = getMasterItemMessagePromise();
        };

        $scope.isCheckboxChecked = function (vendorName) {
            var rtn = false;
            if (vendorName == "-BLANK-") {
                rtn = ($scope.editRow.medVendor.startsWith(",") || $scope.editRow.medVendor.endsWith(",") || $scope.editRow.medVendor.indexOf(",,") > -1 ) ;
            }
            else {
                rtn = ( $scope.editRow.medVendor.trim() == vendorName );
                rtn = rtn || ($scope.editRow.medVendor.endsWith(", " + vendorName));
                rtn = rtn || ($scope.editRow.medVendor.startsWith(vendorName + ","));
                rtn = rtn || ($scope.editRow.medVendor.indexOf(", " + vendorName + ",") > -1);
            }
            return rtn;
        };

        function getMasterItemPromise() {
            return $http.post('/api/v3/getMasterItemFile/', requestData, {timeout: 60000}).then(function(res) {
                var file = new Blob([ res.data ], {
                    type : 'text/tab-separated-values;charset=utf-8a'
                });
                var m = moment().format("YYYYMMDD_hhmm");
                FileSaver.saveAs(file, 'MEDSExtract_exported_' + m + '.csv');
                //var b64 = res.data;
                //blobUtil.base64StringToBlob(b64).then(function (blob) {
                // success
                //    var m = moment().format("YYYYMMDD_hhmm");
                //    FileSaver.saveAs(blob, 'M3Extract_' + m + '.ods');
                //}).catch(function (err) {
                // error
                //    console.log(err);
                //});


            });
        }

        function getMasterItemMessagePromise() {
            return $http.post('/api/v1/getMasterItemMessageFile/', requestData).then(function(res) {
                var file = new Blob([ res.data ], {
                    type : 'text/tab-separated-values;charset=utf-8a'
                });
                var m = moment().format("YYYYMMDD_hhmm");
                FileSaver.saveAs(file, 'M3Message_' + m + '.csv');
                //var b64 = res.data;
                //blobUtil.base64StringToBlob(b64).then(function (blob) {
                // success
                //    var m = moment().format("YYYYMMDD_hhmm");
                //    FileSaver.saveAs(blob, 'M3Extract_' + m + '.ods');
                //}).catch(function (err) {
                // error
                //    console.log(err);
                //});


            });
        }

        function findById(array, value) {
            for (var i = 0; i < array.length; i++) {
                if (array[i]['id'] === value) {
                    return array[i];
                }
            }
            return null;
        }

        function fromUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.dateModified == null) {
                requestData.where.dateModified = {};
            }

            requestData.where.dateModified['$gt'] = date.startOf('day');
            $scope.fromDate = date.format("MM-DD-YYYY");
            getPage();
        }

        function toUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.dateModified == null) {
                requestData.where.dateModified = {};
            }

            requestData.where.dateModified['$lt'] = date.endOf('day');
            $scope.toDate = date.format("MM-DD-YYYY");
            getPage();
        }

        function cleanVar(field) {
            if (field == "") field = null;
            return field;
        }

        function onFilterChanged() {
            var grid = $scope.gridApi.grid;

            for (var i = 0; i < grid.columns.length; i++) {
                var column = grid.columns[i];
                if (column.filters[0].term != null && column.filters[0].term != "") {
                    if (requestData["where"] == null) {
                        requestData["where"] = {};
                    }
                    requestData["where"][column.field] = column.filters[0].term;

                } else {
                    if (requestData["where"] != null && requestData["where"][column.field] != null) {
                        delete requestData["where"][column.field];
                    }
                }
            }
            getPage();
        }

        $scope.forceFilter = function() {
            return onFilterChanged();
        };

        // FRES-128 - add entry to the ActivityLog table
        // Important ! Give at least a couple of seconds if you need to call this function back-to-back
        function addActivityLogEntry(name_table, desc)  {
            console.log("masterItemCtrl.js - addActivityLogEntry : " + name_table + " - " + desc);
            var record = {nameTable:"", description:"", dateModified:0};
            record.dateModified = new Date();
            record.nameTable = name_table;
            record.description = desc;

            return $http.post('/api/recordActivityLog', record).then(function (res) {
                console.log("log recorded");
            }, function (err) {
                console.log("error: " + err);
            });
        }
    }
]);